@@ -8,11 +8,15 @@ import warnings
 from abc import ABC, abstractmethod
 from enum import IntEnum
 from itertools import zip_longest
-from typing import Any, Dict, Iterable, Iterator, List, Optional, Sequence, Set, Tuple, TypeVar, Union
+from typing import (Any, Callable, Dict, Iterable, Iterator, List,
+                    Optional, Sequence, Set, Tuple, TypeVar, Union)
 
 import vapoursynth as vs
 import vsutil
 
+from .dehardsub import hardsub_mask
+from .progress import Progress, BarColumn, FPSColumn, TextColumn, TimeRemainingColumn
+from .render import clip_async_render
 from .util import get_prop
 
 core = vs.core
@@ -480,11 +484,25 @@ def stack_planes(clip: vs.VideoNode, /, stack_vertical: bool = False) -> vs.Vide
         raise ValueError(f"stack_planes: unexpected subsampling {vsutil.get_subsampling(clip)}")
 
 
+def diff_hardsub_mask(a: vs.VideoNode, b: vs.VideoNode, **kwargs: Any) -> vs.VideoNode:
+    """
+    Diff func for :py:func:`lvsfunc.comparison.diff` to use a hardsub mask.
+    This is kinda slow.
+
+    :param a: Clip A
+    :param b: Clip B
+
+    :return:  Diff masked with :py:func:`lvsfunc.dehardsub.hardsub_mask`
+    """
+    return core.std.MaskedMerge(core.std.MakeDiff(a, a), core.std.MakeDiff(a, b), hardsub_mask(a, b))
+
+
 def diff(*clips: vs.VideoNode,
          thr: float = 72,
          height: int = 288,
          return_array: bool = False,
          return_frames: bool = False,
+         diff_func: Callable[[vs.VideoNode, vs.VideoNode], vs.VideoNode] = lambda a, b: core.std.MakeDiff(a, b),
          **namedclips: vs.VideoNode) -> Union[vs.VideoNode, Tuple[vs.VideoNode, List[int]]]:
     """
     Creates a standard :py:class:`lvsfunc.comparison.Stack` between frames from two clips that have differences.
@@ -513,6 +531,8 @@ def diff(*clips: vs.VideoNode,
     :param return_array:  Return frames as an interleaved comparison (using :py:class:`lvsfunc.comparison.Interleave`)
                           (Default: ``False``)
     :param return_frames: Adds `frames list` to the return. (Default: ``False``)
+    :param diff_func:     Function for calculating diff in PlaneStatsMin/Max mode.
+                          (Default: core.std.MakeDiff)
 
     :return: Either an interleaved clip of the differences between the two clips
              or a stack of both input clips on top of MakeDiff clip.
@@ -540,26 +560,40 @@ def diff(*clips: vs.VideoNode,
         nc = list(namedclips.values())
         a, b = vsutil.depth(nc[0], 8), vsutil.depth(nc[1], 8)
 
+    progress = Progress(TextColumn("{task.description}"),
+                        BarColumn(),
+                        TextColumn("{task.completed}/{task.total}"),
+                        TextColumn("{task.percentage:>3.02f}%"),
+                        FPSColumn(),
+                        TimeRemainingColumn())
+
     frames = []
     if thr <= 1:
-        for i, f in enumerate(core.std.PlaneStats(a, b).frames()):
-            print(f"Progress: {i}/{a.num_frames} frames", end="\r")
-            if get_prop(f, 'PlaneStatsDiff', float) > thr:
-                frames.append(i)
+        with progress:
+            for i, f in progress.track(enumerate(core.std.PlaneStats(a, b).frames()),
+                                       description="Diffing clips...",
+                                       total=a.num_frames):
+                if get_prop(f, 'PlaneStatsDiff', float) > thr:
+                    frames.append(i)
         diff = core.std.MakeDiff(a, b)
     else:
-        diff = core.std.MakeDiff(a, b).std.PlaneStats()
-        if diff.format is None:
-            raise ValueError("diff: variable-format clips not supported")  # this is for mypy
-        t = float if diff.format.sample_type == vs.SampleType.FLOAT else int
-        for i, f in enumerate(diff.frames()):
-            print(f"Progress: {i}/{diff.num_frames} frames", end='\r')
-            if get_prop(f, 'PlaneStatsMin', t) <= thr or get_prop(f, 'PlaneStatsMax', t) >= 255 - thr > thr:
-                frames.append(i)
+        diff = diff_func(a, b).std.PlaneStats()
+        assert diff.format is not None
+        t = float if diff.format.sample_type == vs.FLOAT else int
+        with progress:
+            task = progress.add_task("Diffing clips...", total=diff.num_frames)
+
+            def _cb(n: int, f: vs.VideoFrame) -> None:
+                progress.update(task, advance=1)
+                if get_prop(f, 'PlaneStatsMin', t) <= thr or get_prop(f, 'PlaneStatsMax', t) >= 255 - thr > thr:
+                    frames.append(n)
+            clip_async_render(diff, callback=_cb)
 
     if not frames:
         raise ValueError("diff: no differences found")
 
+    frames.sort()
+
     if clips:
         name_a, name_b = "Clip A", "Clip B"
     else:
