@@ -1,49 +1,21 @@
 """
     Wrappers and masks for denoising.
 """
-import math
 from abc import ABC, abstractmethod
 from functools import partial
-from typing import Callable, List, Optional, Tuple, Union
+from typing import List, Optional, Tuple, Union
 
 import vapoursynth as vs
-from vsutil import depth, get_depth, get_y, iterate
+from vsutil import depth, get_y, iterate, join, split
 from vsutil import Range as CRange
 
 from . import util
 from .misc import replace_ranges, scale_thresh
 from .types import Position, Range, Size
 
-try:
-    from cytoolz import functoolz
-except ModuleNotFoundError:
-    try:
-        from toolz import functoolz  # type: ignore
-    except ModuleNotFoundError:
-        raise ModuleNotFoundError("Cannot find functoolz: Please install toolz or cytoolz")
-
 core = vs.core
 
 
-@functoolz.curry
-def adaptive_mask(clip: vs.VideoNode, luma_scaling: float = 8.0) -> vs.VideoNode:
-    """
-    A wrapper to create a luma mask for denoising and/or debanding.
-
-    Function is curried to allow parameter tuning when passing to denoisers
-    that allow you to pass your own mask.
-
-    Dependencies: adaptivegrain
-
-    :param clip:         Input clip
-    :param luma_scaling: Luma scaling factor (Default: 8.0)
-
-    :return:             Luma mask
-    """
-    return core.adg.Mask(clip.std.PlaneStats(), luma_scaling)
-
-
-@functoolz.curry
 def detail_mask(clip: vs.VideoNode, sigma: Optional[float] = None,
                 rad: int = 3, radc: int = 2,
                 brz_a: float = 0.025, brz_b: float = 0.045) -> vs.VideoNode:
@@ -61,8 +33,12 @@ def detail_mask(clip: vs.VideoNode, sigma: Optional[float] = None,
     :param sigma:       Sigma for Bilateral for pre-blurring (Default: False)
     :param rad:         The luma equivalent of gradfun3's "mask" parameter
     :param radc:        The chroma equivalent of gradfun3's "mask" parameter
-    :param brz_a:       Binarizing for the detail mask (Default: 0.025)
-    :param brz_b:       Binarizing for the edge mask (Default: 0.045)
+    :param brz_a:       Binarizing thresh for the detail mask.
+                        Scaled to clip's depth if between 0 and 1 (inclusive),
+                        else assumed to be in native range. (Default: 0.025)
+    :param brz_b:       Binarizing thresh for the edge mask.
+                        Scaled to clip's depth if between 0 and 1 (inclusive),
+                        else assumed to be in native range. (Default: 0.045)
 
     :return:            Detail mask
     """
@@ -76,7 +52,6 @@ def detail_mask(clip: vs.VideoNode, sigma: Optional[float] = None,
             if sigma else clip)
 
     mask_a = range_mask(get_y(blur), rad=rad, radc=radc)
-    mask_a = depth(mask_a, clip.format.bits_per_sample, range=CRange.FULL, range_in=CRange.FULL)
     mask_a = core.std.Binarize(mask_a, brz_a)
 
     mask_b = core.std.Prewitt(get_y(blur))
@@ -87,13 +62,11 @@ def detail_mask(clip: vs.VideoNode, sigma: Optional[float] = None,
     return util.pick_removegrain(mask)(mask, 11)
 
 
-@functoolz.curry
 def halo_mask(clip: vs.VideoNode, rad: int = 2,
-              sigma: float = 1.0, brz: float = 0.35,
-              thmi: int = 80, thma: int = 128,
-              thlimi: int = 50, thlima: int = 100,
-              edgemasking: Callable[[vs.VideoNode, float], vs.VideoNode]
-              = lambda clip, sigma: core.std.Prewitt(clip, scale=sigma)) -> vs.VideoNode:
+              brz: float = 0.35,
+              thmi: float = 0.315, thma: float = 0.5,
+              thlimi: float = 0.195, thlima: float = 0.392,
+              edgemask: Optional[vs.VideoNode] = None) -> vs.VideoNode:
     """
     A halo mask to catch basic haloing, inspired by the mask from FineDehalo.
     Most was copied from there, but some key adjustments were made to center it specifically around masking.
@@ -102,32 +75,30 @@ def halo_mask(clip: vs.VideoNode, rad: int = 2,
     Float made sense for FineDehalo since it uses DeHalo_alpha for dehaloing,
     but the masks themselves use rounded rx/ry values, so there's no reason to bother with floats here.
 
+    All thresholds are float and will be scaled to ``clip``\\'s format.
+    If thresholds are greater than 1, they will be asummed to be in 8-bit and scaled accordingly.
+
     :param clip:            Input clip
     :param rad:             Radius for the mask
-    :param scale:           Multiply all pixels by scale before outputting. Sigma if `edgemask` with a sigma is passed
     :param brz:             Binarizing for shrinking mask (Default: 0.35)
     :param thmi:            Minimum threshold for sharp edges; keep only the sharpest edges
     :param thma:            Maximum threshold for sharp edges; keep only the sharpest edges
     :param thlimi:          Minimum limiting threshold; includes more edges than previously, but ignores simple details
     :param thlima:          Maximum limiting threshold; includes more edges than previously, but ignores simple details
-    :param edgemasking:     Callable function with signature edgemask(clip, scale/sigma)
+    :param edgemask:        Edgemask to use. If None, uses ``clip.std.Prewitt()`` (Default: None).
 
     :return:                Halo mask
     """
-    peak = (1 << get_depth(clip)) - 1
-    smax = _scale(255, peak)
+    smax = scale_thresh(1.0, clip)
 
-    thmi = _scale(thmi, peak)
-    thma = _scale(thma, peak)
-    thlimi = _scale(thlimi, peak)
-    thlima = _scale(thlima, peak)
+    thmi, thma, thlimi, thlima = (scale_thresh(t, clip, assume=8) for t in [thmi, thma, thlimi, thlima])
 
     matrix = [1, 2, 1, 2, 4, 2, 1, 2, 1]
 
-    edgemask = edgemasking(get_y(clip), sigma)
+    edgemask = edgemask or get_y(clip).std.Prewitt()
 
     # Preserve just the strongest edges
-    strong = core.std.Expr(edgemask, expr=f"x {thmi} - {thma-thmi} / {smax} *")
+    strong = core.std.Expr(edgemask, expr=f"x {thmi} - {thlima-thlimi} / {smax} *")
     # Expand to pick up additional halos
     expand = iterate(strong, core.std.Maximum, rad)
 
@@ -147,57 +118,54 @@ def halo_mask(clip: vs.VideoNode, rad: int = 2,
     return core.std.Expr(mask, expr="x 2 *")
 
 
-@functoolz.curry
-def range_mask(clip: vs.VideoNode, rad: int = 2, radc: Optional[int] = None) -> vs.VideoNode:
+def range_mask(clip: vs.VideoNode, rad: int = 2, radc: int = 0) -> vs.VideoNode:
     """
     Min/max mask with separate luma/chroma radii.
 
     rad/radc are the luma/chroma equivalent of gradfun3's "mask" parameter.
     The way gradfun3's mask works is on an 8 bit scale, with rounded dithering of high depth input.
     As such, when following this filter with a Binarize, use the following conversion steps based on input:
-        -  8 bit = Binarize(2) or Binarize(thr_det)
-        - 16 bit = Binarize(384) or Binarize((thr_det - 0.5) * 256)
-        - floats = Binarize(0.005859375) or Binarize((thr_det - 0.5) / 256)
+
+    -  8 bit = Binarize(2) or Binarize(thr_det)
+    - 16 bit = Binarize(384) or Binarize((thr_det - 0.5) * 256)
+    - floats = Binarize(0.005859375) or Binarize((thr_det - 0.5) / 256)
+
     When radii are equal to 1, this filter becomes identical to mt_edge("min/max", 0, 255, 0, 255).
 
     :param clip:    Input clip
     :param rad:     Depth in pixels of the detail/edge masking
-    :param radc:    Chroma equivalent to `rad`
+    :param radc:    Chroma equivalent to ``rad``
 
     :return:        Range mask
     """
-    radc = rad if radc is None else radc
     if radc == 0:
         clip = get_y(clip)
-    ma = _minmax(clip, rad, radc, maximum=True)
-    mi = _minmax(clip, rad, radc, maximum=False)
 
-    return core.std.Expr([ma, mi], expr='x y -')
+    if clip.format is None:
+        raise ValueError("range_mask: 'Variable-format clips not supported'")
+
+    if clip.format.color_family == vs.GRAY:
+        ma = _minmax(clip, rad, True)
+        mi = _minmax(clip, rad, False)
+        mask = core.std.Expr([ma, mi], 'x y -')
+    else:
+        planes = split(clip)
+        for i, rad_ in enumerate([rad, radc, radc]):
+            ma = _minmax(planes[i], rad_, True)
+            mi = _minmax(planes[i], rad_, False)
+            planes[i] = core.std.Expr([ma, mi], 'x y -')
+        mask = join(planes)
+
+    return mask
 
 
 # Helper functions
-def _scale(value: int, peak: int) -> int:
-    x = value * peak / 255
-    return math.floor(x + 0.5) if x > 0 else math.ceil(x - 0.5)
-
-
-def _minmax(clip: vs.VideoNode, sy: int = 2, sc: int = 2, maximum: bool = False) -> vs.VideoNode:
-    yp = sy >= sc
-    yiter = 1 if yp else 0
-    cp = sc >= sy
-    citer = 1 if cp else 0
-    planes = [0] if yp and not cp else [1, 2] if cp and not yp else [0, 1, 2]
-
-    if max(sy, sc) % 3 != 1:
-        coor = [0, 1, 0, 1, 1, 0, 1, 0]
-    else:
-        coor = [1] * 8
-
-    if sy > 0 or sc > 0:
-        func = clip.std.Maximum if maximum else clip.std.Minimum
-        return _minmax(func(planes=planes, coordinates=coor), sy=sy-yiter, sc=sc-citer)
-    else:
-        return clip
+def _minmax(clip: vs.VideoNode, iterations: int, maximum: bool) -> vs.VideoNode:
+    func = core.std.Maximum if maximum else core.std.Minimum
+    for i in range(1, iterations + 1):
+        coord = [0, 1, 0, 1, 1, 0, 1, 0] if (i % 3) != 1 else [1] * 8
+        clip = func(clip, coordinates=coord)
+    return clip
 
 
 class BoundingBox():
@@ -314,7 +282,7 @@ class DeferredMask(ABC):
             for range, rf in zip(self.ranges, self.refframes):
                 mask = depth(self._mask(clip[rf], ref[rf]), clip.format.bits_per_sample,
                              range=CRange.FULL, range_in=CRange.FULL)
-                hm = replace_ranges(hm, mask*len(hm), range)
+                hm = replace_ranges(hm, core.std.Expr([hm, mask*len(hm)], expr="x y max"), range)
 
         return hm if self.bound is None else core.std.MaskedMerge(core.std.BlankClip(hm), hm, bm)
 
