@@ -477,7 +477,7 @@ def colored_clips(amount: int,
 
     Written by Dave <orangechannel@pm.me>.
 
-    :param amount:  Number of ``vapoursynth.VideoNode``s to return
+    :param amount:  Number of ``vapoursynth.VideoNode``\\s to return
     :param max_hue: Maximum hue (0 < hue <= 360) in degrees to generate colors from (uses the HSL color model).
                     Setting this higher than ``315`` will result in the clip colors looping back towards red
                     and is not recommended for visually distinct colors.
@@ -510,20 +510,26 @@ def colored_clips(amount: int,
     return [core.std.BlankClip(color=color, **blank_clip_args) for color in rgb_color_list]
 
 
-def scale_thresh(thresh: float, clip: vs.VideoNode) -> float:
+def scale_thresh(thresh: float, clip: vs.VideoNode, assume: Optional[int] = None) -> float:
     """
     Scale binarization thresholds from float to int.
 
-    :param thresh: Threshold [0, 1]
+    :param thresh: Threshold [0, 1]. If greater than 1, assumed to be in native clip range
     :param clip:   Clip to scale to
+    :param assume: Assume input is this depth when given input >1. If ``None``\\, assume ``clip``\\'s format.
+                   (Default: None)
 
-    :return:       Threshold scaled to [0, 2^clip.depth - 1]
+    :return:       Threshold scaled to [0, 2^clip.depth - 1] (if vs.INTEGER)
     """
     if clip.format is None:
         raise ValueError("scale_thresh: 'Variable-format clips not supported.'")
-    if thresh < 0 or thresh > 1:
-        raise ValueError("scale_thresh: 'Thresholds must be between 0 and 1 (inclusive).'")
-    return thresh if clip.format.sample_type == vs.FLOAT else round(thresh * ((1 << clip.format.bits_per_sample) - 1))
+    if thresh < 0:
+        raise ValueError("scale_thresh: 'Thresholds must be positive.'")
+    if thresh > 1:
+        return thresh if not assume \
+            else round(thresh/((1 << assume) - 1) * ((1 << clip.format.bits_per_sample) - 1))
+    return thresh if clip.format.sample_type == vs.FLOAT or thresh > 1 \
+        else round(thresh * ((1 << clip.format.bits_per_sample) - 1))
 
 
 # TODO: Write function that only masks px of a certain color/threshold of colors.
